/*
  File name: main.cpp
  Copyright: (C) 2005 - 2008 Tomasz Kazmierczak
  Copyright: (C) 2006 Kamil Pawlowski

  Creation date: 27.07.2005
  Last modification date: 19.06.2008

  This file is part of Cytadela

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *

  Application entry point
*/

#include "CCytadelaMain.h"
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdlib.h>

#ifndef DATADIR
#define DATADIR ""
#endif

bool handleArgs(int argc, char *argv[])
{
	if(argc > 1 and (strcmp(argv[1], "-v") == 0 or strcmp(argv[1], "--version") == 0)) {
		fprintf(stdout, "Cytadela, version 0.8\n");
		return false;
	}
	return true;
}

void changeToDataDirectory()
{
	FILE *pFile = fopen("obj/PodziemiaT.3dg", "rb");
	if(pFile) fclose(pFile);
	else chdir(DATADIR);
}

const char *getSysDirectoryPath()
{
	static char szSysDir[FILENAME_MAX];
	char *szHomeDir = getenv("HOME");
	if(szHomeDir)
		sprintf(szSysDir, "%s/.cytadela", szHomeDir);
	else
		sprintf(szSysDir, "sys");

	//check if the directory exists
	DIR *pDir = opendir(szSysDir);
	if(not pDir) {
		//if it does not exist - create it
		//TODO: replace this mess with a standard C function for creating
		//      directories as soon as such finction is added to the C/C++ language
#if defined(__WIN32__)
		if(-1 == mkdir(szSysDir)) {
			fprintf(stderr, "ERROR: could not create \"%s\" directory\n", szSysDir);
			perror(szSysDir);
			return 0;
		}
#else
		if(-1 == mkdir(szSysDir, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH)) {
			fprintf(stderr, "ERROR: could not create \"%s\" directory\n", szSysDir);
			perror(szSysDir);
			return 0;
		}
#endif
	}
	else closedir(pDir);

	return szSysDir;
}

int main(int argc, char *argv[])
{
	fprintf(stdout, "\n");

	//handle the command line arguments
	if(not handleArgs(argc, argv))
		return 0;

	//set current directory to data directory
	changeToDataDirectory();

	CSDLClass *sdl = CSDLClass::instance();
	//first we initialize SDL
	if(not sdl->init()) {
		fprintf(stdout, "Abnormal program termination\n");
		return 1;
	}

	//now we can go into the game
	const char *path = getSysDirectoryPath();
	if(not path) {
		fprintf(stdout, "Abnormal program termination\n");
		return 1;
	}
	CCytadelaMain clMain(path);
	clMain.runMain();

	fprintf(stdout, "\n");
	return 0;
}
